import { Device as _Device } from "./device";
import { Target } from "./target";
import { Texture as _Texture, TextureColorInternalFormat as TexColorFmt, TextureDepthInternalFormat as TexDepthFmt, TextureDepthStencilInternalFormat as TexDepthStencilFmt } from "./texture";
export declare class Framebuffer {
    /**
     * Create a framebuffer containg one or more color buffers with given
     * width and height.
     */
    static withColor(dev: _Device, width: number, height: number, color: _Texture<TexColorFmt> | _Texture<TexColorFmt>[]): Framebuffer;
    /**
     * Create a framebuffer containg a depth buffer with given width and height.
     */
    static withDepth(dev: _Device, width: number, height: number, depth: _Texture<TexDepthFmt>): Framebuffer;
    /**
     * Create a framebuffer containg a depth-stencil buffer with given
     * width and height.
     */
    static withDepthStencil(dev: _Device, width: number, height: number, depthStencil: _Texture<TexDepthStencilFmt>): Framebuffer;
    /**
     * Create a framebuffer containg one or more color buffers and a depth
     * buffer with given width and height.
     */
    static withColorDepth(dev: _Device, width: number, height: number, color: _Texture<TexColorFmt> | _Texture<TexColorFmt>[], depth: _Texture<TexDepthFmt>): Framebuffer;
    /**
     * Create a framebuffer containg one or more color buffers and a
     * depth-stencil buffer with given width and height.
     */
    static withColorDepthStencil(dev: _Device, width: number, height: number, color: _Texture<TexColorFmt> | _Texture<TexColorFmt>[], depthStencil: _Texture<TexDepthStencilFmt>): Framebuffer;
    readonly width: number;
    readonly height: number;
    readonly glFramebuffer: WebGLFramebuffer | null;
    private dev;
    private glColorAttachments;
    private framebufferTarget;
    private colors;
    private depthStencil?;
    private depthOnly;
    private constructor();
    /**
     * Reinitialize invalid framebuffer, eg. after context is lost.
     */
    restore(): void;
    /**
     * Request a render target from this framebuffer to draw into. The target
     * will contain all attached color buffers.
     *
     * Drawing should be done within the callback by
     * calling `ratget.clear()` or `target.draw()` family of methods.
     *
     * Also see `device.target()`.
     */
    target(cb: (rt: Target) => void): void;
    private init();
}
